/**
 * display scheduler status in the assistant frame
 */
var GTXScheduler = {
	/**
	 * scheduler status: suspended
	 */
	STATUS_SUSPENDED:  0,

	/**
	 * scheduler status: running
	 */
	STATUS_RUNNING: 1,

	/**
	 * scheduler status: suspending
	 * scheduler is suspended, but waiting for jobs to finish to
	 * switch to suspended state
	 */
	STATUS_SUSPENDING: 2,
	
	/**
	 * url base for system.node images
	 */
	IMG_URL_BASE: "<?=$STAG;?>&do=11&module=system&img=",
	
	/**
	 * scheduler api url
	 */
	QUERY_URL_BASE: "<?=$STAG;?>&do=875&action=",

	/**
	 * button for status not detected
	 */
	IMG_NONE: "scheduler-status-button-none.gif",

	/**
	 * button for status running
	 */
	IMG_RUNNING: "scheduler-status-button-running.gif",

	/**
	 * button for status suspending
	 */
	IMG_SUSPENDING: "scheduler-status-button-suspending.gif",
	
	/**
	 * button for status suspended
	 */
	IMG_SUSPENDED: "system&img=scheduler-status-button-suspended.gif",
	
	/**
	 * status message for status running
	 */
	MSG_RUNNING: null,

	/**
	 * status message for status suspending
	 */
	MSG_SUSPENDING: null,

	/**
	 * status message for status suspended
	 */
	MSG_SUSPENDED: null,
	
	/**
	 * tracks the current scheduler status
	 */
	status: -1,
	
	/**
	 * will hold a backreference to the instatiated class for solving closure issues
	 */
	ref: null,
	
	/**
	 * keep a reference to the assistant view node
	 * "assview" would have made such a great var name :) 
	 */
	aref: null,
	
	/**
	 * aref's original html code
	 */
	arefCode: null,
	
	/**
	 * if the user has the right to suspend scheduler tasks
	 */
	allowSuspend: false,
	
	/**
	 * init from scheduler status xml api
	 */
	init: function(msg_running, msg_suspending, msg_suspended, allowSuspend) {
		// as one can't include node core functions in js libs i18n strings have to be provided on init
		this.MSG_RUNNING = msg_running;
		this.MSG_SUSPENDING = msg_suspending;
		this.MSG_SUSPENDED = msg_suspended;
		if (allowSuspend === true) {
			this.allowSuspend = true;
		}
		// this is an ugly hack as I was not able to pass the "this" reference
		// through multiple setTimeout and jQuery callbacks
		GTXScheduler.ref = this;
		setInterval(function() {
			GTXScheduler.ref.updateStatus();
		}, 10000);		

		this.updateStatus();
		
		// now insert scheduler status display into the assistant
		// and keep a reference to the view node and it's original html code
		this.aref = jQuery(top.main.ass.document).find("#view");
		this.arefCode = this.aref.html();
		var allowSuspendCode = "";
		if (this.allowSuspend) {
			allowSuspendCode = "onClick=\"top.main.list.GTXScheduler.toggleStatus();\" style=\"cursor: pointer\"";
		}
		this.aref.html("<span id=\"scheduler-status\" " + allowSuspendCode + ">" +
			"<img id=\"scheduler-button\" src=\"" + this.IMG_URL_BASE + this.IMG_NONE + "\" " + 
			"style=\"position: relative; top: 4px;\"> " +
			"<span id=\"scheduler-status-infotext\">Scheduler</span></span>");
		jQuery(window).unload(function() {
			// hide scheduler controls on unload and restore original html code
			GTXScheduler.ref.aref.find("#scheduler-status").fadeOut("normal");
			GTXScheduler.ref.aref.html(GTXScheduler.ref.arefCode);
		});
	},
	
	/**
	 * toggle scheduler status and trigger reaction accordingly
	 */
	toggleStatus: function() {
		switch (this.status) {
			case this.STATUS_RUNNING:
				this.suspend();
				break;
			case this.STATUS_SUSPENDING:
			case this.STATUS_SUSPENDED:
				this.resume();
			break;
		}
	},
	
	/**
	 * suspends the scheduler
	 */
	suspend: function(ignoreIds) {
		jQuery.get(this.QUERY_URL_BASE + "suspend&ignoreJobs=" + ignoreIds);
		GTXScheduler.ref.aref.find("#scheduler-button").attr("src", this.IMG_URL_BASE + this.IMG_SUSPENDING);
		GTXScheduler.ref.aref.find("#scheduler-status-infotext").html(this.MSG_SUSPENDING);
		GTXScheduler.ref.updateStatus();
	},
	
	/**
	 * switches the scheduler back to running state
	 */
	resume: function() {
		jQuery.get(this.QUERY_URL_BASE + "resume");
		GTXScheduler.ref.aref.find("#scheduler-button").attr("src", this.IMG_URL_BASE + this.IMG_RUNNING);
		GTXScheduler.ref.aref.find("#scheduler-status-infotext").html(this.MSG_RUNNING);
		GTXScheduler.ref.updateStatus();
	},
	
	/**
	 * updates the status
	 */
	updateStatus: function() {
		var oldStatus = GTXScheduler.ref.status; 
		
		// our xml response kills jQuery in some weird way, so we have to rely on string matching :( 
		jQuery.get(this.QUERY_URL_BASE + "status", null, function(data) {
			if (data.search("<message>suspended</message>") != -1 && oldStatus != GTXScheduler.ref.STATUS_SUSPENDED) {
				GTXScheduler.ref.aref.find("#scheduler-button").attr("src", GTXScheduler.ref.IMG_URL_BASE + GTXScheduler.ref.IMG_SUSPENDED);
				GTXScheduler.ref.aref.find("#scheduler-status-infotext").html(GTXScheduler.ref.MSG_SUSPENDED);
				GTXScheduler.ref.status = GTXScheduler.ref.STATUS_SUSPENDED;
				// used only in autoupdate UI
				jQuery("#guidesuspendscheduler").attr("class", "done");
				jQuery("#guideresumescheduler").attr("class", "");
			} else if (data.search("<message>suspending</message>") != -1 && oldStatus != GTXScheduler.ref.STATUS_SUSPENDING) {
				GTXScheduler.ref.aref.find("#scheduler-button").attr("src", GTXScheduler.ref.IMG_URL_BASE + GTXScheduler.ref.IMG_SUSPENDING);
				GTXScheduler.ref.aref.find("#scheduler-status-infotext").html(GTXScheduler.ref.MSG_SUSPENDING);
				GTXScheduler.ref.status = GTXScheduler.ref.STATUS_SUSPENDING;
				// used only in autoupdate UI
				jQuery("#guidesuspendscheduler").attr("class", "inprogress");
				jQuery("#guideresumescheduler").attr("class", "inprogress");
			} else if (data.search("<message>running</message>") != -1 && oldStatus != GTXScheduler.ref.STATUS_RUNNING) {
				GTXScheduler.ref.aref.find("#scheduler-button").attr("src", GTXScheduler.ref.IMG_URL_BASE + GTXScheduler.ref.IMG_RUNNING);
				GTXScheduler.ref.aref.find("#scheduler-status-infotext").html(GTXScheduler.ref.MSG_RUNNING);
				GTXScheduler.ref.status = GTXScheduler.ref.STATUS_RUNNING;
				// used only in autoupdate UI
				jQuery("#guidesuspendscheduler").attr("class", "");
				jQuery("#guideresumescheduler").attr("class", "done");
			} else {
				// TODO the improbable case of an error goes here...
			}
			
			// reload if status has changed
			if (oldStatus != -1 && oldStatus != GTXScheduler.ref.status) {
				setTimeout("document.location.href = document.location.href", 2000);
			}
		}, "text");
	}
}